@@ -61,25 +61,7 @@
 
 #include "libmachista.h"
 #include "hashmap.h"
-
-#ifdef __MACH__
-/* Tiger compatibility */
-#ifndef LC_RPATH
-#define LC_RPATH       (0x1c | LC_REQ_DYLD)    /* runpath additions */
-/*
- * The rpath_command contains a path which at runtime should be added to
- * the current run path used to find @rpath prefixed dylibs.
- */
-struct rpath_command {
-    uint32_t     cmd;       /* LC_RPATH */
-    uint32_t     cmdsize;   /* includes string */
-    union lc_str path;      /* path to add to run path */
-};
-#endif
-#ifndef LC_REEXPORT_DYLIB
-#define LC_REEXPORT_DYLIB (0x1f | LC_REQ_DYLD) /* load and re-export dylib */
-#endif
-#endif /* __MACH__ */
+#include "strlcpy.h"
 
 typedef struct macho_input {
     const void *data;
@@ -467,8 +449,12 @@ int macho_parse_file(macho_handle_t *handle, const char *filepath, const macho_t
     /* Open input file */
     if ((fd = open(filepath, O_RDONLY)) < 0) {
 #ifdef HAVE__DYLD_SHARED_CACHE_CONTAINS_PATH
-        if (_dyld_shared_cache_contains_path(filepath)) {
-            return MACHO_ECACHE;
+    /* All systems that have this function in the SDK should also have
+       __builtin_available, so not bothering to check for it. */
+        if (__builtin_available(macos 11.0, ios 14.0, watchos 7.0, tvos 14.0, *)) {
+            if (_dyld_shared_cache_contains_path(filepath)) {
+                return MACHO_ECACHE;
+            }
         }
 #endif /* HAVE__DYLD_SHARED_CACHE_CONTAINS_PATH */
         return MACHO_EFILE;
@@ -505,7 +491,7 @@ int macho_parse_file(macho_handle_t *handle, const char *filepath, const macho_t
             ret = MACHO_EMEM;
         }
     } else {
-        /* An error occured, free mt */
+        /* An error occurred, free mt */
         free_macho_t((macho_t *)*res);
         *res = NULL;
     }
@@ -546,17 +532,8 @@ void macho_destroy_handle(macho_handle_t *handle) {
 
 /* Returns string representation of the MACHO_* error code constants */
 const char *macho_strerror(int err) {
-    int num;
-#ifdef HAVE_FLS
-    num = fls(err);
-#else
-    /* Tiger compatibility, see #42186 */
-    num = 0;
-    while (err > 0) {
-        err >>= 1;
-        num++;
-    }
-#endif
+#ifdef __MACH__
+    int num = fls(err);
 
     static char *errors[] = {
         /* 0x00 */ "Success",
@@ -568,5 +545,8 @@ const char *macho_strerror(int err) {
         /* 0x20 */ "Shared cache only",
     };
     return errors[num];
+#else
+    return "";
+#endif
 }
 
