@@ -41,6 +41,13 @@ namespace eval porttrace {
     # between traced processes and the server-side of trace mode.
     variable fifo
 
+    ##
+    # The mktemp(3) template used to generate a filename for the fifo.
+    # Note that Unix sockets are limited to 109 characters and that the
+    # macports user must be able to connect to the socket (and in case of
+    # non-root installations, the current user, too).
+    variable fifo_mktemp_template "/tmp/macports-trace/sock-XXXXXX"
+
     ##
     # The Tcl thread that runs the server side of trace mode and deals with
     # requests from traced processes.
@@ -59,7 +66,7 @@ namespace eval porttrace {
     proc appendEntry {sandbox path action} {
         upvar 2 $sandbox sndbxlst
 
-        set mapping {}
+        set mapping [list]
         # Escape backslashes with backslashes
         lappend mapping "\\" "\\\\"
         # Escape colons with \:
@@ -122,24 +129,31 @@ namespace eval porttrace {
     # @param workpath The $workpath of the current installation
     proc trace_start {workpath} {
         global \
-            developer_dir distpath env macportsuser os.platform \
-            portpath prefix
+            developer_dir distpath env macportsuser configure.sdkroot \
+            portpath prefix use_xcode
 
         variable fifo
+        variable fifo_mktemp_template
 
         if {[catch {package require Thread} error]} {
             ui_warn "Trace mode requires Tcl Thread package ($error)"
             return 0
         }
 
-        # Select a name for the socket to be used to communicate with the
-        # processes being traced. Note that Unix sockets are limited to 109
-        # characters and that the the macports user must be able to connect to
-        # the socket (and in case of non-root installations, the current user,
-        # too). We're not prefixing the path in /tmp with a separate
-        # macports-specific directory, because the might not be writable by all
-        # users.
-        set fifo "/tmp/macports-trace-[pid]-[expr {int(rand() * 10000)}]"
+        # Generate a name for the socket to be used to communicate with the
+        # processes being traced.
+        set fifo [mktemp $fifo_mktemp_template]
+
+        # Create enclosing directory with correct permissions, i.e. no
+        # finding out what the socket is called by listing the directory,
+        # but it can be opened if you know its name.
+        set fifo_dir [file dirname $fifo]
+        file mkdir $fifo_dir
+        if {[geteuid] == 0} {
+            file attributes $fifo_dir -permissions 0311 -owner $macportsuser
+        } else {
+            file attributes $fifo_dir -permissions 0311
+        }
 
         # Make sure the socket doesn't exist yet (this would cause errors
         # later)
@@ -150,13 +164,13 @@ namespace eval porttrace {
         create_slave $workpath $fifo
 
         # Launch darwintrace.dylib.
-        set tracelib [file join ${portutil::autoconf::tcl_package_path} darwintrace1.0 darwintrace.dylib]
+        set darwintracepath [file join ${portutil::autoconf::tcl_package_path} darwintrace1.0 darwintrace.dylib]
 
         # Add darwintrace.dylib as last entry in DYLD_INSERT_LIBRARIES
         if {[info exists env(DYLD_INSERT_LIBRARIES)] && [string length $env(DYLD_INSERT_LIBRARIES)] > 0} {
-            set env(DYLD_INSERT_LIBRARIES) "${env(DYLD_INSERT_LIBRARIES)}:${tracelib}"
+            set env(DYLD_INSERT_LIBRARIES) "${env(DYLD_INSERT_LIBRARIES)}:${darwintracepath}"
         } else {
-            set env(DYLD_INSERT_LIBRARIES) ${tracelib}
+            set env(DYLD_INSERT_LIBRARIES) ${darwintracepath}
         }
         # Tell traced processes where to find their communication socket back
         # to this code.
@@ -202,11 +216,22 @@ namespace eval porttrace {
             }
         }
 
+        # Allow timezone info & access to system certificates
+        allow trace_sandbox "/var/db/timezone/zoneinfo"
+        allow trace_sandbox "/var/db/mds/system"
+
+        # Allow access to SDK if it's not inside the Developer folder.
+        if {${configure.sdkroot} ne ""} {
+            allow trace_sandbox "${configure.sdkroot}"
+        }
+
         # Allow access to some Xcode specifics
-        allow trace_sandbox "/var/db/xcode_select_link"
-        allow trace_sandbox "/var/db/mds"
-        allow trace_sandbox [file normalize ~${macportsuser}/Library/Preferences/com.apple.dt.Xcode.plist]
-        allow trace_sandbox "$env(HOME)/Library/Preferences/com.apple.dt.Xcode.plist"
+        set xcode_paths [list]
+        lappend xcode_paths "/var/db/xcode_select_link"
+        lappend xcode_paths "/var/db/mds"
+        # XXX Tcl9 unsafe
+        lappend xcode_paths [file normalize ~${macportsuser}/Library/Preferences/com.apple.dt.Xcode.plist]
+        lappend xcode_paths "$env(HOME)/Library/Preferences/com.apple.dt.Xcode.plist"
 
         # Allow access to developer_dir; however, if it ends with /Contents/Developer, strip
         # that. If it doesn't leave that in place to avoid allowing access to "/"!
@@ -214,7 +239,18 @@ namespace eval porttrace {
         if {[llength $ddsplit] > 2 && [lindex $ddsplit end-1] eq "Contents" && [lindex $ddsplit end] eq "Developer"} {
             set ddsplit [lrange $ddsplit 0 end-2]
         }
-        allow trace_sandbox [file join {*}$ddsplit]
+        lappend xcode_paths [file join {*}$ddsplit]
+
+        set cltpath "/Library/Developer/CommandLineTools"
+        if {[tbool use_xcode]} {
+            foreach xcode_path $xcode_paths {
+                allow trace_sandbox $xcode_path
+            }
+        } else {
+            foreach xcode_path $xcode_paths {
+                deny trace_sandbox $xcode_path
+            }
+        }
 
         # Allow launchd.db access to avoid failing on port-load(1)/port-unload(1)/port-reload(1)
         allow trace_sandbox "/var/db/launchd.db"
@@ -230,6 +266,10 @@ namespace eval porttrace {
 
         # Grant access to the directory we use to mirror binaries under SIP
         allow trace_sandbox ${portutil::autoconf::trace_sipworkaround_path}
+        # Grant access to MacPorts' clonebin utilities
+        if {${portutil::autoconf::clonebin_path} ne ""} {
+            allow trace_sandbox ${portutil::autoconf::clonebin_path}
+        }
         # Defer back to MacPorts for dependency checks inside $prefix. This must be at the end,
         # or it'll be used instead of more specific rules.
         ask trace_sandbox $prefix
@@ -247,9 +287,7 @@ namespace eval porttrace {
     # the communication socket. Just must call this once for each call to
     # trace_start.
     proc trace_stop {} {
-        global \
-            env \
-            macosx_version
+        global env
 
         variable fifo
 
@@ -262,6 +300,7 @@ namespace eval porttrace {
         tracelib clean
         # Delete the socket file
         file delete -force $fifo
+        file delete -force [file dirname $fifo]
 
         # Delete the slave.
         delete_slave
@@ -366,29 +405,30 @@ namespace eval porttrace {
         # Create the thread.
         set thread [macports_create_thread]
 
-        # The slave thred needs this file and macports 1.0
-        thread::send $thread "package require porttrace 1.0"
-        thread::send $thread "package require macports 1.0"
+        # The slave thread needs this file and macports 1.0
+        thread::send $thread [list package require porttrace 1.0]
+        thread::send $thread [list package require macports 1.0]
 
         # slave needs ui_{info,warn,debug,error}...
         # make sure to sync this with ../pextlib1.0/tracelib.c!
-        thread::send $thread "macports::ui_init debug"
-        thread::send $thread "macports::ui_init info"
-        thread::send $thread "macports::ui_init warn"
-        thread::send $thread "macports::ui_init error"
+        thread::send $thread [list macports::ui_init debug]
+        thread::send $thread [list macports::ui_init info]
+        thread::send $thread [list macports::ui_init warn]
+        thread::send $thread [list macports::ui_init error]
 
         # and these variables
-        thread::send $thread "set prefix \"$prefix\"; set developer_dir \"$developer_dir\""
+        thread::send $thread [list set prefix ${prefix}]
+        thread::send $thread [list set developer_dir $developer_dir]
         # The slave thread requires the registry package.
-        thread::send $thread "package require registry 1.0"
+        thread::send $thread [list package require registry 1.0]
         # and an open registry
-        thread::send $thread "registry::open [file join ${registry.path} registry registry.db]"
+        thread::send $thread [list registry::open [file join ${registry.path} registry registry.db]]
 
         # Initialize the slave
-        thread::send $thread "porttrace::slave_init $fifo $workpath"
+        thread::send $thread [list porttrace::slave_init $fifo $workpath]
 
         # Run slave asynchronously
-        thread::send -async $thread "porttrace::slave_run"
+        thread::send -async $thread [list porttrace::slave_run]
     }
 
     ##
@@ -406,8 +446,8 @@ namespace eval porttrace {
         set workpath $p_workpath
 
         # Initialize the sandbox violation lists
-        set sandbox_violation_list {}
-        set sandbox_unknown_list {}
+        set sandbox_violation_list [list]
+        set sandbox_unknown_list [list]
 
         # Create the socket
         tracelib setname $fifo
@@ -442,7 +482,7 @@ namespace eval porttrace {
     proc slave_send {command} {
         variable thread
 
-        if {[thread::send $thread "$command" result]} {
+        if {[thread::send $thread $command result]} {
             return -code error "thread::send \"$command\" failed: $result"
         }
         return $result
