@@ -95,7 +95,7 @@ static int list_obj_to_entry(Tcl_Interp* interp, reg_entry*** entries,
  * required. That's OK because there's only one place this function is called,
  * and it's called with all of them there.
  */
-static int entry_create(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[]) {
+static int entry_create(Tcl_Interp* interp, int objc, Tcl_Obj* const objv[]) {
     reg_registry* reg = registry_for(interp, reg_attached);
     if (objc != 7) {
         Tcl_WrongNumArgs(interp, 2, objv, "name version revision variants "
@@ -129,12 +129,13 @@ static int entry_create(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[]) {
  * Deletes an entry from the registry (then closes it). If this is done within a
  * transaction and the transaction is rolled back, the entry will remain valid.
  */
-static int entry_delete(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[]) {
+static int entry_delete(Tcl_Interp* interp, int objc, Tcl_Obj* const objv[]) {
     reg_registry* reg = registry_for(interp, reg_attached);
     if (objc != 3) {
         Tcl_WrongNumArgs(interp, 1, objv, "delete entry");
         return TCL_ERROR;
-    } if (reg == NULL) {
+    }
+    if (reg == NULL) {
         return TCL_ERROR;
     } else {
         reg_error error;
@@ -146,6 +147,17 @@ static int entry_delete(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[]) {
         if (!reg_entry_delete(entry, &error)) {
             return registry_failed(interp, &error);
         }
+        /* We want to delete the command but not free the entry object,
+           since we may want to restore it (delete_entry_list will free
+           it if the transaction is not rolled back). So remove the
+           command's associated deleteProc first. The reg_entry_free
+           below will take care of it if there is no transaction in
+           progress. */
+        Tcl_CmdInfo info;
+        if (Tcl_GetCommandInfo(interp, Tcl_GetString(objv[2]), &info)) {
+            info.deleteProc = NULL;
+            Tcl_SetCommandInfo(interp, Tcl_GetString(objv[2]), &info);
+        }
         /* if there's a transaction going on, record this entry in a list so we
          * can roll it back if necessary
          */
@@ -175,7 +187,7 @@ static int entry_delete(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[]) {
  *
  * Opens an entry matching the given parameters.
  */
-static int entry_open(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[]) {
+static int entry_open(Tcl_Interp* interp, int objc, Tcl_Obj* const objv[]) {
     reg_registry* reg = registry_for(interp, reg_attached);
     if (objc != 7) {
         Tcl_WrongNumArgs(interp, 1, objv, "open portname version revision "
@@ -209,7 +221,7 @@ static int entry_open(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[]) {
  *
  * Closes an entry. It will remain in the registry until next time.
  */
-static int entry_close(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[]) {
+static int entry_close(Tcl_Interp* interp, int objc, Tcl_Obj* const objv[]) {
     if (objc != 3) {
         Tcl_WrongNumArgs(interp, 1, objv, "close entry");
         return TCL_ERROR;
@@ -248,116 +260,92 @@ static strategy_type strategies[] = {
  * Can be given an option of -exact, -glob, -regexp or -null to specify the
  * matching strategy; defaults to exact.
  */
-static int entry_search(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[]) {
-    int i, j;
-    reg_registry* reg = registry_for(interp, reg_attached);
-    if (reg == NULL) {
+static int entry_search(Tcl_Interp* interp, int objc, Tcl_Obj* const *objv) {
+    reg_registry *reg = registry_for(interp, reg_attached);
+    if (!reg) {
         return TCL_ERROR;
+    }
+    const char **keys = malloc(objc * sizeof(char *));
+    const char **vals = malloc(objc * sizeof(char *));
+    int *strats = malloc(objc * sizeof(int));
+    if (!keys || !vals || !strats) {
+        goto cleanup_error;
+    }
+    int key_count = 0;
+
+    /* Skip the "registry::entry" "search" bit */
+    objv += 2;
+    objc -= 2;
+
+    for (int i = 0; i < objc; ++key_count /* i in incremented in the loop */) {
+        int key_index, strategy_index;
+
+        /* Grab the key */
+        if (Tcl_GetIndexFromObj(interp, objv[i], entry_props, "search key",
+            0, &key_index) != TCL_OK) {
+            goto cleanup_error;
+        }
+        keys[key_count] = entry_props[key_index];
+        if (objc <= ++i) {
+            goto wrong_num_args; /* We're missing a strategy or value */
+        }
+
+        /* Set the default strategy, in case we don't find one */
+        strats[key_count] = reg_strategy_exact;
+        /* Try to grab the strategy */
+        if (Tcl_GetIndexFromObjStruct(interp, objv[i], strategies,
+                sizeof(strategy_type), "option", 0, &strategy_index) !=
+                TCL_ERROR) {
+            strats[key_count] = strategies[strategy_index].strategy;
+            if (objc <= ++i && strats[key_count] != reg_strategy_null) {
+                goto wrong_num_args;
+            } else if (strats[key_count] == reg_strategy_null) {
+                /* This doesn't take a value, so start from the top */
+                continue;
+            }
+        }
+
+        /* Grab the value */
+        if (!(vals[key_count] = Tcl_GetString(objv[i++]))) {
+            goto wrong_num_args; /* We're missing a value */
+        }
+    }
+
+    reg_entry **entries;
+    reg_error error;
+    int entry_count;
+    entry_count = reg_entry_search(reg, keys, vals, key_count, strats, &entries,
+        &error);
+    free(keys);
+    free(vals);
+    free(strats);
+
+    if (entry_count >= 0) {
+        int retval;
+        Tcl_Obj* resultObj;
+        Tcl_Obj** objs;
+        if (list_entry_to_obj(interp, &objs, entries, entry_count, &error)){
+            resultObj = Tcl_NewListObj(entry_count, objs);
+            Tcl_SetObjResult(interp, resultObj);
+            free(objs);
+            retval = TCL_OK;
+        } else {
+            retval = registry_failed(interp, &error);
+        }
+        free(entries);
+        return retval;
     } else {
-        char** keys;
-        char** vals;
-        int* strats;
-        int key_count = 0;
-        reg_entry** entries;
-        reg_error error;
-        int entry_count;
-        for (i = 2; i < objc;) {
-            int index, strat_index, val_length;
-            if (Tcl_GetIndexFromObj(interp, objv[i], entry_props, "search key",
-                        0, &index) != TCL_OK) {
-                return TCL_ERROR;
-            }
-
-            /* we ate the key value */
-            i++;
-
-            /* check whether there's a strategy */
-            if (Tcl_GetString(objv[i])[0] == '-'
-                    && Tcl_GetIndexFromObjStruct(interp, objv[i], strategies,
-                        sizeof(strategy_type), "option", 0, &strat_index)
-                    != TCL_ERROR) {
-                /* this key has a strategy specified, eat the strategy parameter */
-                i++;
-
-                if (strategies[strat_index].strategy != reg_strategy_null) {
-                    /* this key must also have a value */
-
-                    if (Tcl_GetStringFromObj(objv[i], &val_length) == NULL) {
-                        Tcl_WrongNumArgs(interp, 2, objv,
-                                "search ?key ?options? value ...?");
-                        return TCL_ERROR;
-                    }
-
-                    i++;
-                }
-            } else {
-                /* this key must also have a value */
-
-                if (Tcl_GetStringFromObj(objv[i], &val_length) == NULL) {
-                    Tcl_WrongNumArgs(interp, 2, objv,
-                            "search ?key ?options? value ...?");
-                    return TCL_ERROR;
-                }
-
-                i++;
-            }
-
-            key_count++;
-        }
-
-        keys = malloc(key_count * sizeof(char*));
-        vals = malloc(key_count * sizeof(char*));
-        strats = malloc(key_count * sizeof(int));
-        if (!keys || !vals || !strats) {
-            return TCL_ERROR;
-        }
-        for (i = 2, j = 0; i < objc && j < key_count; j++) {
-            int strat_index;
-
-            keys[j] = Tcl_GetString(objv[i++]);
-
-            /* try to get the strategy */
-            if (Tcl_GetString(objv[i])[0] == '-'
-                    && Tcl_GetIndexFromObjStruct(interp, objv[i], strategies,
-                        sizeof(strategy_type), "option", 0, &strat_index)
-                    != TCL_ERROR) {
-                /* this key has a strategy specified */
-                i++;
-
-                strats[j] = strategies[strat_index].strategy;
-            } else {
-                /* use default strategy */
-                strats[j] = reg_strategy_exact;
-            }
-
-            if (strats[j] != reg_strategy_null) {
-                vals[j] = Tcl_GetString(objv[i++]);
-            } else {
-                vals[j] = NULL;
-            }
-        }
-        entry_count = reg_entry_search(reg, keys, vals, key_count,
-                strats, &entries, &error);
-        free(keys);
-        free(vals);
-        free(strats);
-        if (entry_count >= 0) {
-            int retval;
-            Tcl_Obj* resultObj;
-            Tcl_Obj** objs;
-            if (list_entry_to_obj(interp, &objs, entries, entry_count, &error)){
-                resultObj = Tcl_NewListObj(entry_count, objs);
-                Tcl_SetObjResult(interp, resultObj);
-                free(objs);
-                retval = TCL_OK;
-            } else {
-                retval = registry_failed(interp, &error);
-            }
-            free(entries);
-            return retval;
-        }
         return registry_failed(interp, &error);
     }
+
+wrong_num_args:
+    Tcl_WrongNumArgs(interp, 2, objv,
+            "search ?key ?options? value ...?");
+cleanup_error:
+    free(keys);
+    free(vals);
+    free(strats);
+    return TCL_ERROR;
 }
 
 /*
@@ -367,7 +355,7 @@ static int entry_search(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[]) {
  * simply checks if the given string is a valid entry object in the current
  * interp. No query to the database will be made.
  */
-static int entry_exists(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[]) {
+static int entry_exists(Tcl_Interp* interp, int objc, Tcl_Obj* const objv[]) {
     reg_error error;
     if (objc != 3) {
         Tcl_WrongNumArgs(interp, 2, objv, "name");
@@ -401,7 +389,7 @@ static int entry_exists(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[]) {
  *
  * TODO: add more arguments (epoch, revision, variants), maybe
  */
-static int entry_imaged(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[]) {
+static int entry_imaged(Tcl_Interp* interp, int objc, Tcl_Obj* const objv[]) {
     reg_registry* reg = registry_for(interp, reg_attached);
     if (objc == 5 || objc > 6) {
         Tcl_WrongNumArgs(interp, 2, objv, "?name ?version ?revision variants???");
@@ -444,7 +432,7 @@ static int entry_imaged(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[]) {
  * installed ports in 'direct' mode. That is, any port which is capable of
  * satisfying a dependency.
  */
-static int entry_installed(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[]){
+static int entry_installed(Tcl_Interp* interp, int objc, Tcl_Obj* const objv[]){
     reg_registry* reg = registry_for(interp, reg_attached);
     if (objc > 3) {
         Tcl_WrongNumArgs(interp, 2, objv, "?name?");
@@ -482,7 +470,7 @@ static int entry_installed(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[]){
  *
  * Returns the port that owns the given filename (empty string if none).
  */
-static int entry_owner(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[]) {
+static int entry_owner(Tcl_Interp* interp, int objc, Tcl_Obj* const objv[]) {
     reg_registry* reg = registry_for(interp, reg_attached);
     int cs = 1;
 
@@ -520,7 +508,7 @@ static int entry_owner(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[]) {
 
 typedef struct {
     char* name;
-    int (*function)(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[]);
+    int (*function)(Tcl_Interp* interp, int objc, Tcl_Obj* const objv[]);
 } entry_cmd_type;
 
 static entry_cmd_type entry_cmds[] = {
@@ -545,7 +533,7 @@ static entry_cmd_type entry_cmds[] = {
  * represents ports too, but not those in the registry.
  */
 int entry_cmd(ClientData clientData UNUSED, Tcl_Interp* interp, int objc,
-        Tcl_Obj* CONST objv[]) {
+        Tcl_Obj* const objv[]) {
     int cmd_index;
     if (objc < 2) {
         Tcl_WrongNumArgs(interp, 1, objv, "cmd ?arg ...?");
