@@ -497,11 +497,15 @@ proc extract_archive_to_imagedir {location} {
                 # The system bsdtar on 10.15 suffers from https://github.com/libarchive/libarchive/issues/497
                 # Later versions fixed that problem but another remains: https://github.com/libarchive/libarchive/issues/1415 
                 global macports::hfscompression
-                if {${hfscompression} && [getuid] == 0 &&
-                        ![catch {macports::binaryInPath bsdtar}] &&
-                        ![catch {exec bsdtar -x --hfsCompression < /dev/null >& /dev/null}]} {
-                    ui_debug "Using bsdtar with HFS+ compression (if valid)"
-                    set unarchive.cmd "bsdtar"
+                if {${hfscompression} && [getuid] == 0} {
+                    ui_debug "Checking for bsdtar supporting HFS+ compression"
+                    set unarchive.cmd [macports::find_tar_with_hfscompression]
+                    if {${unarchive.cmd} eq {}} {
+                        ui_debug "No bsdtar supporting HFS+ compression found"
+                    }
+                }
+                if {${unarchive.cmd} ne {}} {
+                    ui_debug "Using ${unarchive.cmd}"
                     set unarchive.pre_args {-xvp --hfsCompression -f}
                 } else {
                     set tar "tar"
@@ -636,9 +640,6 @@ proc _progress {args} {
 
 proc _get_port_conflicts {port} {
     global registry::tdbc_connection
-    if {![info exists tdbc_connection]} {
-        registry::tdbc_connect
-    }
     variable conflicts_stmt
     if {![info exists conflicts_stmt]} {
         set query {SELECT files.id, files.path, files.actual_path FROM
@@ -724,15 +725,17 @@ proc _activate_contents {port {rename_list {}}} {
     set seendirs [dict create]
     set confirmed_rename_list [list]
     # This is big and hairy and probably could be done better.
-    # First, we need to check the source file, make sure it exists
+    # First, we need to check the source file, make sure it exists.
     # Then we remove the $location from the path of the file in the contents
-    #  list  and check to see if that file exists
-    # Last, if the file exists, and belongs to another port, and force is set
-    #  we remove the file from the file_map, take ownership of it, and
-    #  clobber it
+    #  list and check to see if that file exists.
+    # Last, if the file exists and force is set, we rename the file to a
+    #  non-conflicting name, and update the activated path in the registry
+    #  entry for the current owner (if any) accordingly, which allows
+    #  the port now being activated to create and own the file.
     set todeactivate [dict create]
+    set reg_forced_renames [list]
     try {
-        registry::write {
+        registry::read {
             foreach file $imagefiles {
                 incr progress_step
                 _progress update $progress_step $progress_total_steps
@@ -777,26 +780,31 @@ proc _activate_contents {port {rename_list {}}} {
                     }
                     if {$owner eq {} || ![dict exists $todeactivate $owner]} {
                         if {$force} {
-                            # if we're forcing the activation, then we move any existing
+                            # If we're forcing the activation, then we move any existing
                             # files to a backup file, both in the filesystem and in the
-                            # registry
+                            # registry. But we need to do the registry part later in the
+                            # write transaction so it will be rolled back if anything
+                            # fails. The filesystem part is rolled back in the on error
+                            # clause of the outermost try statement.
                             if {$owner ne {}} {
                                 # Rename all conflicting files for this owner.
-                                set owner_paths [list]
-                                set owner_actual_paths [list]
-                                foreach {path actual_path} [dict get $conflicts_port_to_paths $owner] {
+                                set owner_deactivate_paths [list]
+                                set owner_activate_paths [list]
+                                set owner_backup_paths [list]
+                                foreach pathpair [dict get $conflicts_port_to_paths $owner] {
+                                    lassign $pathpair path actual_path
                                     lappend owner_deactivate_paths $path
                                     if {![catch {::file type $actual_path}]} {
                                         lappend owner_activate_paths $path
                                         set bakfile ${actual_path}${baksuffix}
                                         lappend owner_backup_paths $bakfile
+                                        _progress intermission
                                         ui_warn "File $actual_path already exists.  Moving to: $bakfile."
                                         ::file rename -force -- $actual_path $bakfile
                                         lappend backups $actual_path
                                     }
                                 }
-                                $owner deactivate $owner_deactivate_paths
-                                $owner activate $owner_activate_paths $owner_backup_paths
+                                lappend reg_forced_renames $owner $owner_deactivate_paths $owner_activate_paths $owner_backup_paths
                             } else {
                                 # Just rename this file.
                                 set bakfile ${file}${baksuffix}
@@ -868,6 +876,14 @@ proc _activate_contents {port {rename_list {}}} {
             # Activate it, and catch errors so we can roll-back
 
             try {
+                if {$force} {
+                    # Update registry to reflect renames of any conflicting files that were found
+                    foreach {owner owner_deactivate_paths owner_activate_paths owner_backup_paths} $reg_forced_renames {
+                        $owner deactivate $owner_deactivate_paths
+                        $owner activate $owner_activate_paths $owner_backup_paths
+                    }
+                    unset reg_forced_renames
+                }
                 $port activate $imagefiles
                 _activate_directories $directories $extracted_dir
                 _activate_files [lmap f $files {string cat ${extracted_dir}${f}}] \
